<?php

namespace Drupal\prevent_entity_unpublish;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\entity_reference_integrity\EntityReferenceDependencyManagerInterface;
use Drupal\entity_reference_integrity_enforce\Exception\ProtectedEntityException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Hook into entity deletes and throw an exception to prevent them disappearing.
 */
class EntityPreupdate implements ContainerInjectionInterface {

  /**
   * The dependency manager.
   *
   * @var \Drupal\entity_reference_integrity\EntityReferenceDependencyManagerInterface
   */
  protected $dependencyManager;

  /**
   * The entity type IDs protection is enabled for.
   *
   * @var array
   */
  protected $enabledEntityTypeIds;

  /**
   * Create a DeleteFormAlter object.
   */
  public function __construct(EntityReferenceDependencyManagerInterface $calculator, array $enabled_entity_type_ids) {
    $this->dependencyManager = $calculator;
    $this->enabledEntityTypeIds = $enabled_entity_type_ids;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_reference_integrity.dependency_manager'),
      !is_null($container->get('config.factory')->get('prevent_entity_unpublish.settings')->get('enabled_entity_type_ids')) ? $container->get('config.factory')->get('prevent_entity_unpublish.settings')->get('enabled_entity_type_ids') : []
    );
  }

  /**
   * Implements entity update.
   */
  public function entityUpdate(EntityInterface $entity, FormStateInterface $form_state) {
    if (in_array($entity->getEntityTypeId(), $this->enabledEntityTypeIds, TRUE) && $this->dependencyManager->hasDependents($entity)) {
      $referencing_entities = $this->dependencyManager->getDependentEntities($entity);
      $output = $this->getEntityList($referencing_entities);
      $form_state->setErrorByName('status', t('Can not unpublish "%a" of type "%b" with label "%c" because other <br/> %e is referencing it and the integrity of this entity type is enforced.',
        [
          "%a" => $entity->getEntityTypeId(),
          "%b" => $entity->bundle(),
          "%c" => $entity->label(),
          "%d" => $entity->id(),
          "%e" => $output,
        ]
      ));
    }
  }

  /**
   * Implements get all entity list.
   */
  public function getEntityList($referencing_entities) {
    $build = [];
    if ($referencing_entities) {
      foreach ($referencing_entities as $entity_type_id => $rentities) {
        $build[$entity_type_id]['label'] = [
          '#type' => 'html_tag',
          '#tag' => 'strong',
          '#value' => reset($rentities)->getEntityType()->getLabel(),
        ];
        $build[$entity_type_id]['list'] = [
          '#theme' => 'item_list',
          '#items' => [],
        ];
        foreach ($rentities as $rentity) {
          $build[$entity_type_id]['list']['#items'][] = $rentity->hasLinkTemplate('canonical') ? $rentity->toLink() : $rentity->label();
        }
      }
    }
    $output = \Drupal::service('renderer')->render($build);
    return $output;
  }

}
